# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2010 Allen Lowe <lallenlowe@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import sys
import os
import warnings
import logging
import gtk
import glib
import dbus
import gobject
import locale
import datetime
import Queue

from dexter.frontend.dialogs.about import AboutDialog
from dexter.frontend.dialogs.new_contact import NewContactDialog
from dexter.frontend.dialogs.merge import MergeDialog

from dexter.frontend.widgets.glade import DexterWindow
from dexter.frontend.widgets.entries import SearchBar
from dexter.frontend.widgets.labels import HeaderLabel, DataLabel
from dexter.backend.utilities import ImportThread
import dexter.backend.models as models
import dexter.backend.utilities as utilities
import dexter.backend.ddbus as dexdbus
from dexter.backend.config import DexterConfig

import gettext
from gettext import gettext as _

from dexter.SimpleGtkbuilderApp import SimpleGtkbuilderApp
from dexter.libcontractor import ShareMenu

from dexter.enums import *

class DexterApp(SimpleGtkbuilderApp):
    def __init__(self, datadir, options):
        # Useful variables
        self.datadir = datadir
        self.icons = gtk.icon_theme_get_default()
        self.welcoming = False
        self.q = None
        self.selected_id = None
        self.config = DexterConfig()
        self.has_contractor = False

        # Setup Translations
        self._setup_translations()
        
        # Setup Data
        self._setup_dbus_client()
        self._setup_data()
        
        # Setup Import from CLI
        self._setup_import()


        #run just the newcontact dialog
        if options.newcontact:
            self._purpose = RUN_CONTACT_DIALOG
            self.contact_dialog = NewContactDialog(self.icons, self.datadir)
            self.contact_dialog.standalone = True
        # Run the application.
        elif options.editcontact:
            self._purpose = RUN_CONTACT_DIALOG
            contact = self.GetContactByID(options.editcontact)
            self.contact_dialog = NewContactDialog(self.icons, self.datadir)
            self.contact_dialog.set_contact_edit(contact)
            self.contact_dialog.populate()
            self.contact_dialog.standalone = True
        else:
            myservice = dexdbus.DexterApp()
            self._purpose = RUN_APP

        if options.newcontact:
            SimpleGtkbuilderApp.__init__(self, None, None)
        else:
            SimpleGtkbuilderApp.__init__(self,
                os.path.join(datadir, "ui", "dexter-main.ui"), "dexter")

            self._setup_contact_list()
            self._setup_toolbar()
            self._setup_contractor()
            
            # Load any config items (i.e. saved hpane width)
            self._load_config()

            if not len(self.contacts):
                self._setup_welcome()
            else:
                self._setup_main_screen()
                self.refresh_list()
            if self.files_to_import:
                self._import_vcard_path(self.files_to_import)

    def _setup_contractor(self):
        try:
            self.menu_contractor = ShareMenu(target=None, forstring=False, mime="text/vcard")
            self.menu_contractor.connect("deactivate", self._on_menu_contractor_deactivate)
            self.toolbutton_contractor.set_sensitive(False)
            self.has_contractor = True
        except:
            self.toolbutton_contractor.set_sensitive(False)
            self.has_contractor = False

    def _setup_dbus_client(self):
        self.session_bus = dbus.SessionBus()
        #############TODO: USE SERVICE FILE INSTEAD############################
        try:
            self.dexter_object = self.session_bus.get_object("org.elementary.dexterserver",
                                                        "/org/elementary/dexterserver")
        except:
            try:
                glib.spawn_async([self.datadir + "/../bin/dexter-server"])
            except glib.GError:
                glib.spawn_async(["/usr/bin/dexter-server"])
            import time; time.sleep(0.5)
            self.dexter_object = self.session_bus.get_object("org.elementary.dexterserver",
                                                        "/org/elementary/dexterserver")
        #############TODO: USE SERVICE FILE INSTEAD############################
        self.GetFullList = self.dexter_object.get_dbus_method("GetFullList")
        self.GetContactByID = self.dexter_object.get_dbus_method("GetContactByID")
        self.GetAddresses = self.dexter_object.get_dbus_method("GetAddresses")
        self.GetPhones = self.dexter_object.get_dbus_method("GetPhones")
        self.GetEmails = self.dexter_object.get_dbus_method("GetEmails")
        self.GetIMNicks = self.dexter_object.get_dbus_method("GetIMNicks")
        self.GetNotes = self.dexter_object.get_dbus_method("GetNotes")
        self.CreateContact = self.dexter_object.get_dbus_method("CreateContact", dbus_interface='org.elementary.dexterserver')
        self.DeleteContact = self.dexter_object.get_dbus_method("DeleteContact")
        self.DeleteContacts = self.dexter_object.get_dbus_method("DeleteContacts")
        self.DBCommit = self.dexter_object.get_dbus_method("DBCommit")
        self.dexter_object.connect_to_signal("ContactAdded", self._on_contact_created_dbus_signal, 'org.elementary.dexterserver')
        self.dexter_object.connect_to_signal("ContactsDeleted", self._on_contacts_deleted_dbus_signal, 'org.elementary.dexterserver')
        self.dexter_object.connect_to_signal("ContactsCleared", self._on_contacts_cleared_dbus_signal, 'org.elementary.dexterserver')
        self.dexter_object.connect_to_signal("WindowRequested", self._on_window_requested_dbus_signal, 'org.elementary.dexterserver')
        self.dexter_object.connect_to_signal("ContactDialogRequested", self._on_contact_dialog_requested_dbus_signal, 'org.elementary.dexterserver')
        self.dexter_object.connect_to_signal("EditContactRequested", self._on_edit_contact_requested_dbus_signal, 'org.elementary.dexterserver')
        self.dexter_object.connect_to_signal("CancelImportRequested", self._on_cancel_import_requested_dbus_signal, 'org.elementary.dexterserver')
        self.dexter_object.connect_to_signal("VcardImportRequested", self._on_vcard_import_requested_dbus_signal, 'org.elementary.dexterserver')
        self.dexter_object.connect_to_signal("ServerClosed", self._on_server_closed_dbus_signal, 'org.elementary.dexterserver')

    # Private Functions
    def _setup_translations(self):
        gettext.bindtextdomain("dexter", "/usr/share/locale")
        gettext.textdomain("dexter")
        try:
            locale.setlocale(locale.LC_ALL, "")
        except Exception:
            print "setlocale failed"

    def _setup_import(self):
        self.last_contact_imported = None
        self.files_to_import = []
        for arg in sys.argv:
            if os.path.exists(arg):
                if arg.endswith(".vcard") or arg.endswith(".vcf"):
                    self.files_to_import.append(arg)

    def _setup_data(self):
        """get a list of contacts from the database"""
        self.contacts = self.GetFullList()
        self.importing = False

    def _setup_contact_list(self):
        self.menuitem_sort_last_name.data = "last_name"
        self.menuitem_sort_first_name.data = "first_name"
        self.menuitem_sort_recent.data = "recent"
         #Contact tree ListStore
        self.contact_model = gtk.ListStore(gobject.TYPE_STRING)
        self.contact_model.set_column_types(str, int, str)
        #Contact list TreeView
        self.cellrenderer_contact_name = self.builder.get_object("cellrenderer_contact_name")
        self.treeview_contacts.set_model(self.contact_model)
        self.treeviewcolumn_contact.add_attribute(self.cellrenderer_contact_name, 'text', 0)
        self.right_click_menu = self.builder.get_object("right_click_menu")
        self.contact_model.set_sort_column_id(2, gtk.SORT_ASCENDING)
        self.contact_selection = self.treeview_contacts.get_selection()
        self.contact_selection.set_mode(gtk.SELECTION_MULTIPLE)
        self.contact_selection.connect("changed", self._on_row_selected)
        self.window_main.set_focus(self.treeview_contacts)

    def _setup_toolbar(self):
        self._setup_search()

    def _setup_search(self):
        """setup the necessary fields for live searching"""
        self.toolbutton1 = self.builder.get_object("toolbutton1")
        self.search_bar = SearchBar(_("Type to search..."),
                                           self.dexter_object)
        self.toolbutton1.add(self.search_bar)
        self.search_bar.show()
        self.search_bar.connect("changed", self._on_search_bar_changed)
        self.search_bar.connect("icon-press", self._on_search_icon_pressed)
        self.window_main.add_accel_group(self.search_bar.agr)

    def _setup_welcome(self):
        if self.hpaned_main in self.vbox_main.get_children():
            self.vbox_main.remove(self.hpaned_main)
        self.vbox_main.pack_start(self.vbox_welcome)
        self.vbox_welcome.show_all()
        self.search_bar.unhint()
        self.search_bar.set_sensitive(False)
        self.menuitem_sort_by.set_sensitive(False)
        self.window_main.set_focus(self.treeview_contacts)
        self.welcoming = True

    def _setup_main_screen(self):
        if self.vbox_welcome in self.vbox_main.get_children():
            self.vbox_main.remove(self.vbox_welcome)
        self.vbox_main.pack_start(self.hpaned_main)
        self.menuitem_sort_by.set_sensitive(True)
        self.window_main.set_focus(self.treeview_contacts)
        self.welcoming = False

    def _load_config(self):
        hpane_width = int(self.config.get("general", "hpane_width"))
        self.hpaned_main.set_position(hpane_width)

    def _save_config(self):
        hpane_width = self.hpaned_main.get_position()
        self.config.set("general", "hpane_width", hpane_width)


    # Callbacks
    def _on_contact_created_dbus_signal(self, contact):
        if self.welcoming:
            self._setup_main_screen()
        if not self.importing:
            self.refresh_list()
            self.select_by_id(int(contact["id"]))
        self.num_contacts.set_text(" %i %s" % (len(self.contact_model), _("Contacts")))

    def _on_contacts_deleted_dbus_signal(self, contact_ids):
        self.refresh_list()

    def _on_contacts_cleared_dbus_signal(self):
        if not self.vbox_welcome in self.vbox_main.get_children():
            self._setup_welcome()

    def _on_window_requested_dbus_signal(self):
        self.window_main.present()

    def _on_contact_dialog_requested_dbus_signal(self):
        self._on_toolbutton_new_contact_clicked(None)

    def _on_edit_contact_requested_dbus_signal(self, uid):
        self.edit_contact(uid)

    def _on_cancel_import_requested_dbus_signal(self):
        self.q.request_stop = True

    def _on_vcard_import_requested_dbus_signal(self, paths):
        self._import_vcard_path(paths)

    def _on_server_closed_dbus_signal(self):
        self._setup_dbus_client()

    def _on_window_main_delete_event(self, widget, event):
        # Clean up code for saving application state should be added here.
        self.DBCommit()
        self._save_config()
        gtk.main_quit()

    def _on_menuitem_quit_activated(self, widget):
        self._on_window_main_delete_event(None, None)

    def _on_search_bar_changed(self, widget):
        """live search everytime the text in the searchbar changes"""
        self.search_bar.search()
        text = self.search_bar.get_text()
        # this is a workaround for a bug, don't delete it again
        if text[1:] == self.search_bar.hint_string:
            text = text[0]
            self.search_bar.unhint(text)
            self.search_bar.searching = True
        if self.search_bar.searching:
            if not text:
                self.refresh_list(self.contacts)
                self.window_main.set_focus(self.treeview_contacts)
                self.search_bar.searching = False
            else:
                self.refresh_list(self.search_bar.result_set)
            with warnings.catch_warnings():
                warnings.simplefilter("ignore")
                self.treeview_contacts.set_cursor_on_cell((0,))
            
    def _on_search_icon_pressed(self, widget, icon_pos, event):
        """search_bar clear button is clicked"""
        if icon_pos == gtk.ENTRY_ICON_SECONDARY:
            self.refresh_list(self.contacts)
            self.window_main.set_focus(self.treeview_contacts)
            self.clear_contact_viewer()

    def _on_treeview_contacts_key_pressed(self, widget, event):
        """a keyboard key is pressed in the contact list treeview
        
        if text, send focus to search_bar and insert first character
        if Return, edit the selected contact
        """
        if gtk.gdk.keyval_name(event.keyval) == "Return":
            if self.toolbutton_edit_contact.get_sensitive():
                self._on_toolbutton_edit_contact_clicked(self.toolbutton_edit_contact)
        if gtk.gdk.keyval_name(event.keyval) == "Delete":
            self._on_toolbutton_delete_contact_clicked(None)
        
        if event.string.isalnum():
            #need to consider if focus should not be given to search_bar,
            #instead just sending the text to it each time
            if self.search_bar.get_sensitive():
                self.window_main.set_focus(self.search_bar)
                self.search_bar.insert_text(event.string, -1)
                self.search_bar.set_position(-1)

    def _on_treeview_contacts_clicked(self, widget, event):
        """user right, or double clicks in the contact list treeview"""
        if event.type == gtk.gdk._2BUTTON_PRESS:  # double click
            if self.toolbutton_edit_contact.get_sensitive():
                self._on_toolbutton_edit_contact_clicked(self.toolbutton_edit_contact)
        elif event.button == 3:  # right click
            x = int(event.x)
            y = int(event.y)
            time = event.time
            pthinfo = self.treeview_contacts.get_path_at_pos(x, y)
            if pthinfo is not None:
                path, col, cellx, celly = pthinfo
                self.treeview_contacts.grab_focus()
                if self.contact_selection.count_selected_rows() < 2:
                    self.treeview_contacts.set_cursor( path, col, 0)
                    self.menuitem_right_edit.set_sensitive(True)
                    self.menuitem_right_merge.set_sensitive(False)
                else:
                    self.menuitem_right_edit.set_sensitive(False)
                    self.menuitem_right_merge.set_sensitive(True)
                self.right_click_menu.popup( None, None, None, event.button, time)
            return True

    def _on_toolbutton_actions_clicked(self, widget):
        """user clicked the app menu button"""
        if not self.menu_actions.get_attach_widget():
            self.menu_actions.attach_to_widget(widget, None)
        self.menu_actions.popup(None, None, self.menu_position, 1, gtk.get_current_event_time())
        self.menu_actions.select_first(True)

    def _on_toolbutton_contractor_clicked(self, widget):
        """user clicked the app menu button"""
        contacts = self.get_selected_contacts()
        if contacts:
            if len(contacts) == 1:
                if contacts[0]["last_name"]:
                    filename = contacts[0]["first_name"].lower().replace(' ', '') + "-" + contacts[0]["last_name"].lower() + ".vcf"
                else:
                    filename = contacts[0]["first_name"].lower().replace(' ', '') + ".vcf"
            else:
                filename = "contacts.vcf"
            filename = "/tmp/" + filename
            utilities.export_to_vcard(contacts, filename, self.dexter_object)
            self.menu_contractor.update_target(filename)
        else:
            self.menu_contractor.update_target(None)
        if not self.menu_contractor.get_attach_widget():
            self.menu_contractor.attach_to_widget(widget, None)
        self.menu_contractor.popup(None, None, self.menu_position, 1, gtk.get_current_event_time())
        self.menu_contractor.select_first(True)

    def _on_menuitem_select_all_activated(self, widget):
        self.contact_selection.select_all()

    def _on_menuitem_help_activated(self, widget):
        try:
            glib.spawn_async(["/usr/bin/xdg-open",
                              "https://answers.launchpad.net/dexter-rolodex"])
        except:
            gtk.show_uri(None, "https://answers.launchpad.net/dexter-rolodex",
                         gtk.gdk.CURRENT_TIME)

    def _on_menuitem_translate_activated(self, widget):
        try:
            glib.spawn_async(["/usr/bin/xdg-open",
                           "https://translations.launchpad.net/dexter-rolodex"])
        except:
            gtk.show_uri(None, "https://translations.launchpad.net/dexter-rolodex",
                         gtk.gdk.CURRENT_TIME)

    def _on_menuitem_report_bug_activated(self, widget):
        try:
            glib.spawn_async(["/usr/bin/xdg-open",
                              "https://bugs.launchpad.net/dexter-rolodex"])
        except:
            gtk.show_uri(None, "https://bugs.launchpad.net/dexter-rolodex",
                         gtk.gdk.CURRENT_TIME)

    def _on_menuitem_merge_activated(self, widget):
        result = utilities.merge_contacts(self.get_selected_contacts(), self.dexter_object)
        if type(result) == dict:
            merge = MergeDialog()
            merge.set_transient_for(self.window_main)
            merge.populate(result)
            response = merge.run()
            if response == gtk.RESPONSE_OK:
                result = merge.data
            merge.destroy()
            id_number = self.CreateContact(result["first"][0], result["middle"][0],
                                           result["last"][0], result["org"][0],
                                           result["birthday"][0], result["photo"],
                                           result["emails"], result["phones"],
                                           result["addresses"], result["imnicks"], result["notes"],)
        elif type(result) == int:
            id_number = result
        else:
            logging.error("data from merge was neither an id number or a dict of data")
            id_number = None
        self._on_store_changed()
        self.select_by_id(id_number)

    def _on_menuitem_export_activated(self, widget):
        """user selects the export to vcard action"""
        contacts = self.get_selected_contacts()
        chooser = gtk.FileChooserDialog(title=None,
                                        action=gtk.FILE_CHOOSER_ACTION_SAVE,
                                        buttons=(gtk.STOCK_CANCEL,
                                                 gtk.RESPONSE_CANCEL,
                                                 gtk.STOCK_SAVE,
                                                 gtk.RESPONSE_OK))
        chooser.set_default_response(gtk.RESPONSE_OK)
        chooser.set_current_folder(os.path.expanduser("~")) # set to users home folder
        if len(contacts) == 1:
            if contacts[0]["last_name"]:
                chooser.set_current_name(contacts[0]["first_name"].lower().replace(' ', '') + "-" + 
                                         contacts[0]["last_name"].lower() + ".vcf")
            else:
                chooser.set_current_name(contacts[0]["first_name"].lower().replace(' ', '') + ".vcf")
        else:
            chooser.set_current_name("contacts.vcf")
        response = chooser.run()
        if response == gtk.RESPONSE_OK:
            utilities.export_to_vcard(contacts, chooser.get_filename(), self.dexter_object)
        chooser.destroy()

    def _on_welcome_import_clicked(self, widget):
        self._on_menuitem_import_activated(widget)
        
    def _on_menuitem_import_activated(self, widget):
        """user selects the import from vcard action"""
        chooser = gtk.FileChooserDialog(title=None,
                                        action=gtk.FILE_CHOOSER_ACTION_OPEN,
                                        buttons=(gtk.STOCK_CANCEL,
                                                 gtk.RESPONSE_CANCEL,
                                                 gtk.STOCK_OPEN,
                                                 gtk.RESPONSE_OK))
        chooser.set_default_response(gtk.RESPONSE_OK)
        chooser.set_current_folder(os.path.expanduser("~"))
        chooser.set_select_multiple(True)
        filter = gtk.FileFilter()
        filter.set_name(_("vCards"))
        filter.add_pattern("*.vcf")  # only allow choosing .vcf files
        filter.add_pattern("*.vcard")  # or .vcard files
        chooser.add_filter(filter)
        response = chooser.run()
        if response == gtk.RESPONSE_OK:
            vcard_paths = chooser.get_filenames()
            self._import_vcard_path(vcard_paths)
        chooser.destroy()

    def _import_vcard_path(self, paths):
        self.q = Queue.Queue() # create a queue for the thread to pass data back
        self.q.request_stop = False
        import_thread = ImportThread(paths, self.q)
        import_thread.start() # start parsing vcard in a separate thread
        self.timer = gobject.timeout_add(100, self.import_timeout)#create a timer for adding contacts, so as to not block the GUI
        timer2 = gobject.timeout_add(200, self.update_timeout)# a timer to make sure the UI gets refreshed sometimes during import
        self.vbox_progress.show()
        self.label_progress.set_text(_("Importing Contacts..."))
        self.hbox_progress.show()
        self.progressbar.show()
        self.menuitem_import.set_sensitive(False)
        self.search_bar.unhint()
        self.search_bar.set_sensitive(False)
        self.clear_contact_viewer()

    def _on_button_photo_clicked(self, widget):
        """photo button clicked, open photo in external viewer"""
        photo_name = self.get_selected_contact()["photo"]
        try:
            glib.spawn_async(["/usr/bin/xdg-open", models.IMAGE_DIR + \
                              str(photo_name)])
        except:
            gtk.show_uri(None, "file://"+models.IMAGE_DIR+str(photo_name),
                         gtk.gdk.CURRENT_TIME)

    def _on_button_cancel_clicked(self, widget):
        """ cancel button next to progress bar clicked"""
        if self.q:
            self.q.request_stop = True

    def _on_store_changed(self):
        """run when a contact has been created, edited, merged, or deleted"""
        self.DBCommit()
        self.search_bar.hint()
        self.clear_contact_viewer()
        self.search_bar.result_sets = {}
        self.search_bar.result_set = []

    def _on_toolbutton_delete_contact_clicked(self, widget):
        """remove selected contact when the delete button is clicked"""
        selected_contacts = self.get_selected_contacts()
        if len(selected_contacts) < 2:
            primary = _("Are you sure you want to delete the selected contact?")
            secondary = _("You will not be able to recover this contact at a later date.")
        else:
            primary = _("Are you sure you want to delete the selected contacts?")
            secondary = _("You will not be able to recover these contacts at a later date.")
        md = gtk.MessageDialog(None, gtk.DIALOG_DESTROY_WITH_PARENT,
                               gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO,
                              primary)
        md.format_secondary_text(secondary)
        response = md.run()
        md.destroy()
        if response == gtk.RESPONSE_YES:
            ids = []
            for contact in selected_contacts:
                ids.append(int(contact["id"]))
            self.DeleteContacts(ids)
            self._on_store_changed()

    def _on_welcome_create_clicked(self, widget):
        self._on_toolbutton_new_contact_clicked(widget)

    def _on_toolbutton_new_contact_clicked(self, widget):
        """open a dialog to create a new contact"""
        new = NewContactDialog(self.icons, self.datadir)
        new.dialog.set_transient_for(self.window_main)
        response = new.run()
        id_number = new.new_contact_id
        new.dialog.destroy()
        if response == gtk.RESPONSE_OK:
            self._on_store_changed()
            if self.welcoming:
                self._setup_main_screen()

    def _on_toolbutton_edit_contact_clicked(self, widget):
        """open a dialog to edit the selected contact"""
        contact = self.get_selected_contact()
        self.edit_contact(None, contact)
        

    def _on_sort_changed(self, widget):
        """user selects on of the radio buttons from the sort menu"""
        selected = None
        group = widget.get_group()
        for item in group:
            if item.get_active():
                selected = item.data
        if selected == "last_name":
            self.contact_model.set_sort_column_id(2, gtk.SORT_ASCENDING)
        elif selected == "first_name":
            self.contact_model.set_sort_column_id(0, gtk.SORT_ASCENDING)
        elif selected == "recent":
            self.contact_model.set_sort_column_id(1, gtk.SORT_DESCENDING)

    def _on_row_selected(self, widget, data=None):
        """update the contact viewing area when a contact is selected
        
        also set the proper sensitivity based on the number of selected contacts
        """
        contact = self.get_selected_contact()
        self.clear_contact_viewer()
        if contact:
            if self.contact_selection.count_selected_rows() == 1:
                self.toolbutton_edit_contact.set_sensitive(True)
                if self.has_contractor:
                    self.toolbutton_contractor.set_sensitive(True)
                self.menuitem_merge.set_sensitive(False)
            else:
                self.toolbutton_edit_contact.set_sensitive(False)
                if self.has_contractor:
                    self.toolbutton_contractor.set_sensitive(True)
                self.menuitem_merge.set_sensitive(True)
            self.toolbutton_delete_contact.set_sensitive(True)
            self.menuitem_export.set_sensitive(True)
            self.populate_viewer(contact)
        else:
            if not self.importing:
                if not len(self.contact_model):
                    self.label_name.set_text(_("No Contacts Found"))
                else:
                    self.clear_contact_viewer()
            else:
                self.clear_contact_viewer()

    def _on_menu_actions_deactivate(self, widget):
        self.toolbutton_actions.set_active(False)

    def _on_menu_contractor_deactivate(self, widget):
        self.toolbutton_contractor.set_active(False)

    def _on_menuitem_about_activated(self, widget, data=None):
        """Display the About Dialog"""
        about = AboutDialog()
        response = about.run()
        about.destroy()

    # Public Functions
    def edit_contact(self, uid=None, contact=None):
        if not contact:
            contact = self.GetContactByID(uid)
        edit = NewContactDialog(self.icons, self.datadir)
        edit.dialog.set_transient_for(self.window_main)
        edit.set_contact_edit(contact)
        edit.populate()
        response = edit.run()
        id_number = edit.new_contact_id
        edit.dialog.destroy()
        if response == gtk.RESPONSE_OK:
            self._on_store_changed()
            self.select_by_id(id_number)

    def menu_position(self, menu, data=None):
        widget = menu.get_attach_widget()
        allocation = widget.get_allocation()
        window_pos = widget.window.get_position()
        x = window_pos[0] + allocation.x
        y = window_pos[1] + allocation.y + allocation.height
        return (x, y, True)

    def update_timeout(self):
        if self.timer:
            if gtk.events_pending():
                gtk.main_iteration(True)
            return True
        else:
            return False
        
    def import_timeout(self):
        """timer that adds one item from the queue to the database.
        
        runs at an interval until the queue is empty
        """
        if not self.q.request_stop:
            try:
                item = self.q.get(True, 2)
                new_id = self.CreateContact(item[0], item[1], item[2], item[3],
                                            item[4], item[5], item[6], item[7],
                                            item[8], item[9], item[10])
                self.last_contact_imported = new_id
                contact = self.GetContactByID(new_id)
                self.progressbar.pulse()
                self.num_contacts.set_text(" %i %s" % (len(self.contact_model), _("Contacts")))
                self.importing = True
                return True
            except Queue.Empty:
                if not self.importing:
                    md = gtk.MessageDialog(None, gtk.DIALOG_DESTROY_WITH_PARENT,
                                           gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE,
                                           _("Could not import vCard file"))
                    md.run()
                    md.destroy()
                self.finish_importing()
                return False
        else:
            self.finish_importing()
            return False

    def finish_importing(self):
        self.progressbar.hide()
        self.hbox_progress.hide()
        self.vbox_progress.hide()
        self.label_progress.set_text("")
        self.progressbar.set_fraction(0.0)
        self.DBCommit()
        self.importing = False
        self.refresh_list()
        if self.last_contact_imported:
            self.select_by_id(self.last_contact_imported)
        self.search_bar.hint()
        self.search_bar.set_sensitive(True)
        self.timer = None
        self.menuitem_import.set_sensitive(True)

    def refresh_list(self, contact_list="empty"):
        """
        refresh the contact list with whatever list of contact 
        objects is passed in. If no list is passed in it will 
        get a list from the database
        """
        if contact_list == "empty":
            self.contacts = self.GetFullList()
            contact_list = self.contacts
        elif not contact_list:
            if not self.importing:
                self.label_name.set_text(_("No Contacts Found"))
            else:
                self.clear_contact_viewer()
        self.search_bar.refresh(len(self.contacts)) # give searchbar the updated contact numbers
        self.contact_model.clear()
        for contact in contact_list:
            self.insert_in_list(contact)
        self.num_contacts.set_text(" %i %s" % (len(self.contact_model), _("Contacts")))
        #clearing the list unselects any contacts, so make these items insensitive
        self.toolbutton_edit_contact.set_sensitive(False)
        self.toolbutton_contractor.set_sensitive(False)
        self.menuitem_merge.set_sensitive(False)
        self.toolbutton_delete_contact.set_sensitive(False)
        self.menuitem_export.set_sensitive(False)
        if not len(self.contacts):
            if not self.vbox_welcome in self.vbox_main.get_children():
                self._setup_welcome()

    def insert_in_list(self, contact):
        self.contact_model.append([contact["full_name"], int(contact["id"]),
                                   contact["last_name"]])

    def remove_from_list(self, contact_ids=[]):
        row_iters = []
        for row in self.contact_model:
            if row[1] in contact_ids:
                rowiter = self.contact_model.get_iter(row.path)
                row_iters.append(rowiter)
        for rowiter in row_iters:
            self.contact_model.remove(rowiter)

    #this is still here for compatibility with code I haven't updated yet
    def get_selected_contact(self):
        """get the contact that is currently selected from the database"""
        contact = None
        selection = self.contact_selection.get_selected_rows()
        pathlist = selection[1]
        try:
            path = pathlist[0]
        except:
            return contact
        treeiter = self.contact_model.get_iter(path)
        try:
            id = selection[0].get_value(treeiter, 1)
            contact = self.GetContactByID(id)
        except TypeError:
            contact = None
        return contact
        
    def get_selected_contacts(self):
        """get a list of contacts that are currently selected from the database"""
        contactlist = []
        selection = self.contact_selection.get_selected_rows()
        for path in selection[1]:
            treeiter = self.contact_model.get_iter(path)
            try:
                id = selection[0].get_value(treeiter, 1)
                contact = self.GetContactByID(id)
            except TypeError:
                contact = None
            contactlist.append(contact)
        return contactlist

    def select_by_id(self, id_number):
        """select the contact in the contact list treeview based on the contacts id"""
        for row in self.contact_model:
            if id_number == row[1]:
                self.treeview_contacts.set_cursor_on_cell(row.path)

    def clear_contact_viewer(self):
        """empty the contents of the contact viewer"""
        self.label_name.set_text("")
        self.label_organization.set_text("")
        self.button_photo.hide()
        self.image_photo.set_from_icon_name("stock_person", 48)
        for row in self.vbox_column_header.get_children():
            self.vbox_column_header.remove(row)
        for row in self.vbox_column_data.get_children():
            self.vbox_column_data.remove(row)
        self.toolbutton_edit_contact.set_sensitive(False)
        self.toolbutton_contractor.set_sensitive(False)
        self.menuitem_merge.set_sensitive(False)
        self.toolbutton_delete_contact.set_sensitive(False)
        self.menuitem_export.set_sensitive(False)

    def blank_line(self):
        """add a blank line to the contact viewer"""
        self.vbox_column_header.pack_start(gtk.Label(),
                                      expand=False, fill=True, padding=0)
        self.vbox_column_data.pack_start(gtk.Label(),
                                    expand=False, fill=True, padding=0)

    def populate_viewer(self, contact):
        """fill the contents of the contact viewer with the given contact"""
        note_space = 0
        self.label_name.set_text(contact["full_name"])
        if contact["organization"] != "None":
            self.label_organization.set_text(contact["organization"])
        else:
            self.label_organization.set_text("")
        if contact["photo"] != "None":
            self.button_photo.show()
            pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(models.IMAGE_DIR +
                                                          str(contact["photo"]),
                                                          60, 60)
            self.image_photo.set_from_pixbuf(pixbuf)
        else:
            self.button_photo.hide()
        if contact["birthday"] != "None":
            birthdaylist = contact["birthday"].split("-")
            birthday = datetime.datetime(int(birthdaylist[0]),
                                             int(birthdaylist[1]),
                                             int(birthdaylist[2]))
            self.vbox_column_header.pack_start(HeaderLabel(_("Birthdate")),
                                          expand=False, fill=True, padding=0)
            self.vbox_column_data.pack_start(DataLabel(
                        birthday.strftime("%B %d, %Y")),
                        expand=False, fill=True, padding=0)
            self.blank_line()
        emails = self.GetEmails(int(contact["id"]))
        if len(emails):
            for email in emails:
                email_label = DataLabel(email[0])
                email_label_markup = "<a href=\"%s\" title=\"%s\">%s</a>" % \
                                     ("mailto:"+email[0].strip(),
                                     _("Send email..."),email[0].strip())
                email_label.set_markup(email_label_markup)
                self.vbox_column_header.pack_start(HeaderLabel(email[1]),
                                              expand=False, fill=True, padding=0)
                self.vbox_column_data.pack_start(email_label,
                                              expand=False, fill=True, padding=0)
            self.blank_line()
        phones = self.GetPhones(int(contact["id"]))
        if len(phones):
            for phone in phones:
                self.vbox_column_header.pack_start(HeaderLabel(phone[1]),
                                              expand=False, fill=True, padding=0)
                self.vbox_column_data.pack_start(DataLabel(phone[0]),
                                              expand=False, fill=True, padding=0)
            self.blank_line()
        addresses = self.GetAddresses(int(contact["id"]))
        if len(addresses):
            for address in addresses:
                full_address = ""
                multi_line_address = ""
                spaced = False
                adr_label = DataLabel("")
                if address["type"] != "None":
                    self.vbox_column_header.pack_start(
                                       HeaderLabel(address["type"]),
                                       expand=False, fill=True, padding=0)
                else:
                    self.vbox_column_header.pack_start(
                                       HeaderLabel(_("Address")),
                                       expand=False, fill=True, padding=0)
                if address["street"] != "None":
                    full_address = address["street"]
                    multi_line_address = address["street"]
                    spaced = True
                else:
                    streetlabel = None
                text = ""
                if address["city"] != "None":
                    text += "%s, " % (address["city"])
                if address["state"] != "None":
                    text += "%s " % (address["state"])
                if address["zip"] != "None":
                    text += "%s" % (address["zip"])
                if text:
                    if spaced:
                        self.vbox_column_header.pack_start(gtk.Label(),
                                             expand=False, fill=True, padding=0)
                        multi_line_address += "\n" + text
                        note_space += 3
                    else:
                        multi_line_address = text
                    full_address += " " + text
                adr_label_markup = "<a href=\"%s\" title=\"%s\">%s</a>" % \
                ("http://maps.google.com/maps?q="+"+".join(full_address.strip().split()),
                _("Search on Google maps..."), multi_line_address)
                adr_label.set_markup(adr_label_markup)
                self.vbox_column_data.pack_start(adr_label,
                                            expand=False, fill=True, padding=0)
            self.blank_line()
        for note in self.GetNotes(int(contact["id"])):
            if note.strip() != "None" and note.strip():
                self.vbox_column_header.pack_start(HeaderLabel(_("Note")),
                                              expand=False, fill=True, padding=0)
                self.vbox_column_data.pack_start(DataLabel(note), expand=False,
                                                 fill=True, padding=note_space)
        self.vbox_column_header.show_all()
        self.vbox_column_data.show_all()


    # Public Functions
    def run(self):
        if self._purpose == RUN_APP:
            self.window_main.show()
        elif self._purpose == RUN_CONTACT_DIALOG:
            self.contact_dialog.run()
        
        super(DexterApp, self).run()
