# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2010 Allen Lowe <lallenlowe@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import glib
import gtk
import dbus


class APIError(Exception):
    def __init__(self, value):
        self.value = value
    def __str__(self):
        return repr(self.value)


class Contract(object):

    def __init__(self):
        self._setup_dbus_client()

    def _setup_dbus_client(self):
        self.session_bus = dbus.SessionBus()
        self.contractor_object = self.session_bus.get_object("org.elementary.contractor",
                                                             "/org/elementary/contractor")
        self.GetAllServices = self.contractor_object.get_dbus_method("GetAllServices")
        self.GetServicesByMime = self.contractor_object.get_dbus_method("GetServicesByMime")
        self.GetServicesForString = self.contractor_object.get_dbus_method("GetServicesForString")
        self.contractor_object.connect_to_signal("ServerClosed", self._on_server_closed_dbus_signal, 'org.elementary.contractor')

    def _on_server_closed_dbus_signal(self):
        self._setup_dbus_client()


class ShareItem(gtk.MenuItem):

    def __init__(self, appdata, target=None):
        self.appdata = appdata
        self.target = target
        gtk.MenuItem.__init__(self, label=self.appdata["Description"])
        self.connect("activate", self._on_activate)
        
        if not self.target:
            self.set_sensitive(False)

    def update_target(self, target):
        if target:
            self.set_sensitive(True)
            self.target = target
        else:
            self.set_sensitive(False)

    def _on_activate(self, widget):
        exec_string = self.appdata["Exec"] % (self.target)
        exec_pieces = str(exec_string).split()
        glib.spawn_async(exec_pieces, flags=glib.SPAWN_SEARCH_PATH)

class ShareMenu(gtk.Menu):

    def __init__(self, target=None, forstring=False, mime=None):
        gtk.Menu.__init__(self)
        results = []
        self.target = target
        self.contract = Contract()
        if not forstring and not mime:
            raise APIError("not set to use string and no mimetype specified")
        elif forstring:
            results = self.contract.GetServicesForString()
        else:
            results = self.contract.GetServicesByMime(mime)
        
        for result in results:
            self.append(ShareItem(result, self.target))
        self.show_all()

    def update_target(self, target):
        self.target = target
        for child in self.get_children():
            child.update_target(self.target)
